<?php

namespace HighwayPro\App\Data\Model\UrlExtra;

use HighWayPro\Original\Collections\Collection;
use HighWayPro\Original\Collections\Mapper\Types;
use HighWayPro\Original\Data\Model\Domain;
use Highwaypro\App\Data\Model\Preferences\PostPreferences;
use Highwaypro\App\Data\Model\Preferences\UrlPreferences;

Class UrlExtra extends Domain
{
    const DEFAULT_CONTEXT = 'post';
    const EXTRA_TRACKING_SCRIPT_MODE_REPLACE = 'replace';
    const EXTRA_TRACKING_SCRIPT_MODE_APPEND = 'append';

    public static function fields()
    {
        return new Collection([
            'id' => Types::INTEGER,
            'url_id' => Types::INTEGER,
            'name' => Types::STRING,
            'value' => Types::STRING()->escape(Types::returnValueCallable()),
            // 'value' will be overridden by static::valueMaps()
            'type' => Types::STRING,
            'context' => Types::COLLECTION
        ]);   
    }

    public static function valueMaps()
    {
        return new Collection([
            'keyword_injection_keywords' => Types::STRING()->escape(Types::returnValueCallable()),
            'keyword_injection' => Types::STRING()->escape(Types::returnValueCallable()),
            'keyword_injection_limit' => Types::INTEGER()->withDefault(-1),

            'link_placement_click_behaviour' => Types::STRING()->escape(Types::returnValueCallable()),
            'link_placement_jit_redirection_is_enabled' => PostPreferences::fields()->get('link_placement_jit_redirection_is_enabled'),
            'link_placement_title_attribute' => Types::STRING()->escape(Types::returnValueCallable()),

            /*--rel attrs--*/
            'link_placement_follow_type' => PostPreferences::fields()->get('link_placement_follow_type'),
            'link_placement_noreferrer_is_enabled' => PostPreferences::fields()->get('link_placement_noreferrer_is_enabled'),
            'link_placement_sponsored_is_enabled' => PostPreferences::fields()->get('link_placement_sponsored_is_enabled'),
            'link_placement_ugc_is_enabled' => PostPreferences::fields()->get('link_placement_ugc_is_enabled'),
            /*--rel attrs--*/

            'http_redirection_status_code' => UrlPreferences::fields()->get('http_redirection_status_code'),
            'http_referrer_type' => UrlPreferences::fields()->get('http_referrer_type'),

            'http_no_index_mode' => UrlPreferences::fields()->get('http_no_index_mode'),

            'http_parameter_forwarding_is_enabled' => UrlPreferences::fields()->get('http_parameter_forwarding_is_enabled'),
            'http_parameter_forwarding_mode' => UrlPreferences::fields()->get('http_parameter_forwarding_mode'),
            'http_parameter_forwarding_parameter_names' => UrlPreferences::fields()->get('http_parameter_forwarding_parameter_names'),

            'extra_tracking_script_mode' => Types::STRING()->withDefault(UrlExtra::EXTRA_TRACKING_SCRIPT_MODE_APPEND)
                                                           ->allowed([
                                                               UrlExtra::EXTRA_TRACKING_SCRIPT_MODE_REPLACE,
                                                               UrlExtra::EXTRA_TRACKING_SCRIPT_MODE_APPEND
                                                           ]),
            'extra_tracking_scripts' => Types::STRING()->escape(Types::returnValueCallable())
        ]);
    }

    public static function getMapFromValues(array $values)
    {
        (object) $fields = self::fields();

        if (!empty($values['name'])) {
            $fieldType = static::valueMaps()->get($values['name']);

            $fieldType && $fields->set('value', $fieldType);
        }

        return $fields;   
    }
    

    public function map(array $values)
    {
        return static::getMapFromValues($values)->asArray();
    }
    
    public static function validUpdateableFields()
    {
        return [
            'keyword_injection_keywords', 
            'keyword_injection_context', 
            'keyword_injection_limit',


            'link_placement_click_behaviour',
            'link_placement_jit_redirection_is_enabled',

            'link_placement_follow_type',
            'link_placement_noreferrer_is_enabled',
            'link_placement_sponsored_is_enabled',
            'link_placement_ugc_is_enabled',

            'link_placement_title_attribute',

            'http_redirection_status_code',
            'http_referrer_type',
            'http_no_index_mode',

            'http_parameter_forwarding_is_enabled',
            'http_parameter_forwarding_mode',
            'http_parameter_forwarding_parameter_names',
            'extra_tracking_scripts'
        ];
    }
    

    public static function defaultEntities()
    {
        return static::valueMaps()->mapWithKeys(function($type, $name) {
            return [
                'key' => (string) $name,
                'value' => new static([
                    'name' => $name
                ])
            ];
        })->getValues();
    }
 
    public static function getDefaultEntityWithName($extraName)
    {
        return static::defaultEntities()->find(function(UrlExtra $urlExtra) use ($extraName) {
            return $urlExtra->name->is($extraName);
        });   
    }
    

    protected function beforeInsertion()
    {
        if ($this->context->haveNone()) {
            $this->context->push(static::DEFAULT_CONTEXT);
        }
    }        
          
}