<?php

namespace HighwayPro\App\Data\Model\UrlExtra\Validators;

use HighWayPro\Original\Characters\StringManager;
use HighWayPro\Original\Collections\Collection;
use HighwayPro\App\Data\Model\UrlExtra\Exceptions\InvalidKeywordsTypeException;
use HighwayPro\App\Data\Model\UrlExtra\Exceptions\InvalidRedirectionStatusCodeException;
use HighwayPro\App\Data\Model\UrlExtra\Exceptions\InvalidReferrerTypeException;
use Highwaypro\App\Data\Model\Preferences\UrlPreferences;

Class UrlExtraValidator
{
    protected $fields;
    
    public function __construct(Array $fields)
    {
        $this->fields = new Collection($fields);   
    }

    public function validateInjectionKeywords()
    {
        if (!($this->fields->get('keyword_injection_keywords') instanceof StringManager)) {
            (string) $type = gettype($this->fields->get('keyword_injection_keywords'));

            throw new InvalidKeywordsTypeException("Invalid keywords type, received: {$this->fields->get('keyword_injection_keywords')} ({$type})");
        }
    }   

    public function validateRefererType()
    {
        if ($this->fields->get('http_referrer_type')->isNotEither([UrlPreferences::REFERER_TYPE_ALWAYS, 
                                                                   UrlPreferences::REFERER_TYPE_ORIGIN, 
                                                                   UrlPreferences::REFERER_TYPE_HIDE])) {
            throw new InvalidReferrerTypeException("Invalid http_referrer_type, received: {$this->fields->get('http_referrer_type')}");
        }
    }
 
    public function validateRedirectionStatusCode()
    {
        /*mixed*/ $statusCode = (integer) ((string) $this->fields->get('http_redirection_status_code'));

        if ($statusCode < 300 || $statusCode > 399) {
            throw new InvalidRedirectionStatusCodeException("Invalid http_redirection_status_code, redirection status codes must be in the 300-399 range. Received: {$this->fields->get('http_redirection_status_code')}");
        }
    }    
          
}