<?php

namespace HighWayPro\App\HighWayPro\DestinationComponents;

use HighWayPro\App\HighWayPro\HTTP\Request;
use HighWayPro\Original\Characters\StringManager;
use HighWayPro\Original\Collections\Collection;
use HighWayPro\Original\Environment\Env;
use HighwayPro\App\Data\Model\Destinations\DestinationComponentDomain;
use HighwayPro\App\Data\Model\Urls\Url;
use Highwaypro\App\Data\Model\Preferences\UrlPreferences;

Abstract Class DestinationTargetComponent extends DestinationComponent
{
    const type = 'targets';
    const event = 'plugins_loaded';

    /**
     * Returns the Response object to use for the request
     * @return HighWayPro\App\HighWayPro\HTTP\Response
     */
    abstract public function response(); #: Response
    abstract protected function getRawURL(); #: (string)
    /**
     * Called before PHP shuts down. Great for dispatching non-essential works like database/file writes.
     * @return void
     */
    /*abstract*/ public function onShutDown(DestinationComponentDomain $destinationComponentDomain){}

    public static function description()
    {
        return __('The final location the short URL should redirect to.', Env::textDomain());
    }

    public function getUrl()
    {
        (string) $targetURL = (string) $this->getRawURL();

        $targetURL = $this->addForwardedParameters($targetURL);

        if ($this->parameters->queryString->hasValue()) {

            (string) $queryString = ($this->parameters->queryString[0] === '?')? substr($this->parameters->queryString, 1) : $this->parameters->queryString;

            $targetURL = add_query_arg(
                (new Collection(explode('&', $queryString)))->mapWithKeys(function($value, $key) {
                    list($key, $value) = explode('=', $value);
                    return [
                        'key' => $key,
                        'value' => urlencode($value)
                    ];
                })->asArray(),
                $targetURL
            );
        }

        return new StringManager($targetURL);
    }

    protected function addForwardedParameters($targetURL)
    {
        (object) $url = $this->destination->getUrl();

        if ($url->parameterForwardingIsEnabled()) {
            (object) $queryParameters = Request::instance()->unMappedData;
            (object) $allowedQueryParameters = new Collection([]);

            switch ($url->getParameterForwardingMode()) {
                case UrlPreferences::PARAMETER_FORWARDING_MODE_ALL:
                    $allowedQueryParameters = $queryParameters;
                    break;
                case UrlPreferences::PARAMETER_FORWARDING_MODE_INCLUDE_ONLY:
                    $allowedQueryParameters = $this->getAllowedQueryParameters($queryParameters, $url);
                    break;
                case UrlPreferences::PARAMETER_FORWARDING_MODE_EXCLUDE:
                    $allowedQueryParameters = $queryParameters->except($this->getAllowedQueryParameters($queryParameters, $url)->getKeys());
                    break;
            }

            $targetURL = add_query_arg($allowedQueryParameters->map([StringManager::class, 'stringToNative'])->asArray(), $targetURL);
        }

        return $targetURL;
    }

    protected function getAllowedQueryParameters(Collection $queryParameters, Url $url)
    {
        return $queryParameters->filter(function($value, $realParameterName) use ($url) {
            return $url->getParameterForwardingParameterNames()->contain(function($parameterNameToInclude) use ($realParameterName){
                $realParameterName = StringManager::create($realParameterName)->trim()->toLowerCase();
                $parameterNameToInclude = StringManager::create($parameterNameToInclude)->trim()->toLowerCase();

                if ($parameterNameToInclude->endsWith('*')) {
                    return $realParameterName->startsWith($parameterNameToInclude->removeRight('*'));
                }

                return $realParameterName->is($parameterNameToInclude);
            });
        });  
    }
    
}