<?php

namespace HighWayPro\App\HighWayPro\Texts\Dashboard;

use HighWayPro\App\HighWayPro\Texts\TextComponent;
use HighWayPro\Original\Collections\Collection;
use HighWayPro\Original\Environment\Env;
use Highwaypro\App\Data\Model\Preferences\UrlPreferences;

Class PreferencesText extends TextComponent
{
    const name = 'preferences';

    protected static function registerTexts()
    {
        return new Collection([
            'preferencesTitle' => esc_html__('HighWayPro Preferences', Env::textDomain()),

            'overrideNotice' => esc_html__('Some of these settings may be overridden individually per URL. When this happens, these default settings will no longer apply to those URLs.', Env::textDomain()),
            'url' => [
                'autogeneratedUrlEnableLabel' => esc_html__('Create a short URL for each new or updated post', Env::textDomain()),

                'autoGeneratedPathType' => [
                    'title' => esc_html__('Auto Generated Path Type', Env::textDomain()),
                    'description' => esc_html__('Alphabetical: letters only. Numeric: numbers only. Alphanumeric: letters and/or numbers.', Env::textDomain())
                ],
                'autoGeneratedPathAlphanumericCase' => [
                    'title' => esc_html__('Auto generated path case', Env::textDomain()),
                    'description' => esc_html__('Any: a mix of both upper and lower case letters. Ignored when paths are numeric only.', Env::textDomain())
                ],
                'autoGeneratedPathLength' => [
                    'title' => esc_html__('Auto generated path length', Env::textDomain()),
                    'description' => esc_html__('Automatically generated paths with less than 5 characters are not recommended.', Env::textDomain())
                ],
                'redirection' => [
                    'title' => esc_html__('Redirection Preferences', Env::textDomain()),
                ],
                'http_redirection_status_code' => [
                    'title' => esc_html__('Redirection Type (status)', Env::textDomain()),
                    'description' => esc_html__('The HTTP status code to send when the redirection is performed. Due to caching, a 302 (Found) status code is sent by default. Other status codes, like a 301 (Moved Permanently) may make the browser cache the target URL and re-use it if the short URL is clicked twice or more by the same user, which would prevent HighWayPro from tracking and cloaking aditional clicks made by the same user.', Env::textDomain())
                ],
                'http_referrer_type' => [
                    'label' => esc_html__('Referrer URL to send to the target URL.', Env::textDomain()),
                    'description' => esc_html__('The referrer URL that is set when performing the redirection. By default, the page where the URL was clicked will be passed as the referrer URL.', Env::textDomain())
                ],
                'http_no_index_mode' => [
                    'label' => esc_html__('No-index URLs.', Env::textDomain()),
                    'description' => esc_html__('Tell search engines not to index all or some of your URLs by setting a noindex header on each URL. It is up to the search engines to follow this suggestion.', Env::textDomain())
                ],
                'http_no_index_type_ids' => [
                    'label' => esc_html__('URL types to no-index', Env::textDomain()),
                    'description' => esc_html__('Only URLs with these types will be set as noindex.', Env::textDomain())
                ],
                'http_parameter_forwarding_is_enabled' => [
                    'label' => esc_html__('Enable parameter forwarding', Env::textDomain()),
                    'description' => esc_html__('Sets the query string parameters from the short URL to the target URL. if both the short and the target URL have a query parameter with the same name, the parameter from the short URL will be used.', Env::textDomain())
                ],
                'http_parameter_forwarding_mode' => [
                    'label' => esc_html__('Forward:', Env::textDomain()),
                    'description' => esc_html__('Select what parametersto pass to the target URL.', Env::textDomain())
                ],
                'http_parameter_forwarding_parameter_names' => [
                    'label' => esc_html__('Parameter names to __ACTION__:', Env::textDomain()),
                    'action' => [
                        UrlPreferences::PARAMETER_FORWARDING_MODE_INCLUDE_ONLY => esc_html__('include', Env::textDomain()),
                        UrlPreferences::PARAMETER_FORWARDING_MODE_EXCLUDE => esc_html__('exclude', Env::textDomain())
                    ],
                    'description' => esc_html__('A list of parameter names to __ACTION__.__N__ Partial names can be defined by using a * at the end of the name, for example: utm_* will forward all parameters that start with utm_, like utm_source and utm_medium. The wildcard *  may only be specified at the end of the name.', Env::textDomain())
                ]
            ],
            'trackingScripts' => [
                'hp_global_tracking_scripts' => [
                    'label' => esc_html__('Tracking Scripts HTML:', Env::textDomain()),
                    'description' => esc_html__('The HTML of the tracking scripts, like Facebook\'s Pixel code.', Env::textDomain())
                ],
            ],
            'post' => [
                'dynamicLinkInsertion' => [
                    'label' => esc_html__('Enable Dynamic Link Insertion', Env::textDomain()),
                     'description' => esc_html__('Shuts down the insertion on links for all urls and posts.', Env::textDomain()),
                ],
                'keyword_injection_limit' => [
                    'title' => esc_html__('Maximum number of insertions per keyword:', Env::textDomain()),
                    'description' => esc_html__('This limit is per keyword. Once the limit has been reached, no more links will be inserted for the remaining occurrences of that specific keyword. This is a default option. You can override this option on a per-url basis.', Env::textDomain())
                ],
                'keyword_injection_post_types_enabled' => [
                    'title' => esc_html__('Enable insertion on these post types:', Env::textDomain()),
                    'description' => esc_html__('This is a default option. You can override this option on a per-url basis.', Env::textDomain())
                ],
                'link_placement_click_behaviour' => [
                    'title' => esc_html__('When short links inside your content are cliked:', Env::textDomain()),
                    'description' => esc_html__('This is a default option. You can override this option on a per-url basis.', Env::textDomain())
                ],
                'link_placement_follow_type' => [
                    'title' => esc_html__('Default Follow Type:', Env::textDomain()),
                    'description' => esc_html__('Do-follow tells search engines to follow and pass link juice to the short link. This is a default option. You can override this option on a per-url basis.', Env::textDomain())
                ],
                'link_placement_jit_redirection_is_enabled' => [
                    'label' => esc_html__('Enable Just-In-Time redirections', Env::textDomain()),
                     'description' => esc_html__('Shows the first real target URL and swaps it with the short URL just after a visitor clicks on the link. This way the user sees the real URL but gets directed to the short URL when they click it.', Env::textDomain()),
                ],
                'link_placement_follow_type' => [
                    'label' => esc_html__('No Follow', Env::textDomain()),
                    'description' => esc_html__('Mark this link as nofollow. Used by search engines.', Env::textDomain()),
                    'globaldescription' => esc_html__('Mark all links as nofollow by default. Used by search engines.', Env::textDomain()),
                ],
                'link_placement_noreferrer_is_enabled' => [
                    'label' => esc_html__('Hide Referrer', Env::textDomain()),
                    'description' => esc_html__('Prevent the target website to know the page this link was clicked from. This setting onlY works when the url is clicked from a link found within your site content, like an article or a page.', Env::textDomain()),
                ],
                'link_placement_sponsored_is_enabled' => [
                    'label' => esc_html__('Sponsored', Env::textDomain()),
                    'description' => esc_html__('Tells search engines this link is an ad, a paid placement or an affiliate link.', Env::textDomain()),
                ],
                'link_placement_ugc_is_enabled' => [
                    'label' => esc_html__('User-generated content', Env::textDomain()),
                    'description' => esc_html__('Tells search engines this link was placed by a non-trusted user, for example, by a user in a post comment.', Env::textDomain()),
                ],
            ],
            'social' => [
                'og_url_is_enabled' => [
                    'label' => __('Use short urls on social media.', Env::textDomain()),
                    'description' => __('Sets the og:url property to the short url of your posts instead of the original permalink if the post has a short url. Recommended for tracking your social media visitors but adviced against if you want to keep your previous share count.', Env::textDomain()),
                ],
                'og_url_post_types_enabled' => [
                    'title' => __("Set short urls on social media only for these post types", Env::textDomain())
                ]
            ],
            'analytics' => [
                'defaultPreset' => [
                    'label' => __('Default preset to show analytics data.', Env::textDomain()),
                    'description' => __('Select the default period of time to use for clicks data.', Env::textDomain()),
                ],
                'defaultUnit' => [
                    'label' => __("Default unit for the preset", Env::textDomain()),
                    'description' => __("'Hours' is only available in same-day presets (today, yesterday).", Env::textDomain()),
                ]
            ],
            'privacy' => [
                'propertyToGetUserIP' => [
                    'title' => __('How to get the user\'s IP.', Env::textDomain()),
                    'description' => __('Change the defaults if your site is using CloudFlare or Sucuri.', Env::textDomain()),
                ],
            ],
            'dashboard' => [
                'tourIsEnabled' => [
                    'label' => __('Retake the tour.', Env::textDomain()),
                    'description' => __('Click on "Start Tour" if you need to retake the tour.', Env::textDomain()),
                    'button' => __('Start Tour.', Env::textDomain()),
                ],
                'state:urls:sidebar:orderType' => [
                    'NEWEST' => __('Newest first', Env::textDomain()),
                    'OLDEST' => __('Oldest first', Env::textDomain()),
                    'GROUP_BY_TYPE' => __('Group By Type', Env::textDomain()),
                ]
            ]
        ]);
    }
    
}